/*
 * Filesystem resource functionality.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/paging.h>

/* Interface support. */

#include <fsserver/resource.h>

/* Implemented interfaces. */

#include <fsserver/filesystem_object_interface.h>



/* A user filesystem class. */

class FilesystemResource : public Resource, public FilesystemObject
{
protected:
  /* Paging object coordination. */

  Paging *_paging;

public:
  explicit FilesystemResource(Paging *paging = 0)
  : Resource(), _paging(paging)
  {
  }

  /* Server details. */

  int expected_items();

  ipc_server_handler_type handler();

  void *interface()
  { return static_cast<FilesystemObject *>(this); }

  /* Operation methods. */

  virtual long getuserfs(ipc_varg_sys_uid_t,
                         ipc_varg_sys_gid_t,
                         ipc_varg_sys_mode_t,
                         l4_cap_idx_t *)
  {
    return -L4_EACCESS;
  }

  virtual long getuserfs(sys_uid_t, sys_gid_t, sys_mode_t, l4_cap_idx_t *)
  {
    return -L4_EACCESS;
  }
};
