/*
 * Opener context resource functionality.
 *
 * Copyright (C) 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "opener_context_object_server.h"
#include "opener_context_resource.h"



/* Server details. */

int OpenerContextResource::expected_items()
{
  return OpenerContextObject_expected_items;
}

ipc_server_handler_type OpenerContextResource::handler()
{
  return (ipc_server_handler_type) handle_OpenerContextObject;
}



/* In each case, a filesystem object structure is initialised to provide details
   of the filename buffer. Since the dataspace will not be relinquished (and is
   actually this object), a "null" dataspace is given. */

/* Delegate filesystem acquisition to the recorded opener object. */

long OpenerContextResource::getfs(l4_cap_idx_t *fs)
{
  fs_object_t fsobj;

  fsdesc_init_buffer(&fsobj.buffer, (char *) buffer(), this->size());
  return _opener->getfs_object(&fsobj, fs);
}



/* Delegate mounting to the recorded opener object. */

long OpenerContextResource::mount(l4_cap_idx_t fs)
{
  fs_object_t fsobj;

  fsdesc_init_buffer(&fsobj.buffer, (char *) buffer(), this->size());
  return _opener->mount_object(&fsobj, fs);
}



/* Delegate file opening to the recorded opener object. */

long OpenerContextResource::open(int flags, size_t *size, l4_cap_idx_t *file)
{
  fs_object_t fsobj;

  fsdesc_init_buffer(&fsobj.buffer, (char *) buffer(), this->size());
  return _opener->open_object(&fsobj, flags, size, file);
}
